/* cdft.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 
#include "brent.h"

/* ********************************************************************** */

// T Distribution

/*                              Method */

/*     Formula  26.5.27  of   Abramowitz   and  Stegun,   Handbook   of */
/*     Mathematical Functions  (1966) is used to reduce the computation */
/*     of the cumulative distribution function to that of an incomplete */
/*     beta. */

/*     Computation of other parameters involve a seach for a value that */
/*     produces  the desired  value  of P.   The search relies  on  the */
/*     monotinicity of P with the other parameter. */

/* ********************************************************************** */

int cdflib_tCheckX(char * fname, double x)
{
	// Nothing to do.
	return CDFLIB_OK;
}
int cdflib_tCheckParams(char * fname, double v)
{
	int status;
	/*     V > 0 */
	status=cdflib_checkgreaterthan(fname, v, "v", 0.);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	return CDFLIB_OK;
}
int cdflib_tcdf(double x, double v, int lowertail, double *p)
{
	int status;
	double infinite = cdflib_infinite();
	double q;
	status=cdflib_tCheckX("cdflib_tcdf", x);
	if (status!=CDFLIB_OK)
	{
		return status;
	}

	// Check V
	status=cdflib_tCheckParams("cdflib_tcdf", v);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	if (cdflib_isnan(x) || cdflib_isnan(v))
	{
		*p = x+v;
		status = CDFLIB_OK;
		return status;
	}
	if (v==infinite)
	{
		cdflib_cumnor(x, p, &q);
		if (lowertail==CDFLIB_UPPERTAIL)
		{
			*p=q;
		}
		status = CDFLIB_OK;
		return status;
	}
	cdflib_cumt(x, v, p, &q);
	if (lowertail==CDFLIB_UPPERTAIL)
	{
		*p=q;
	}
	status = CDFLIB_OK;
	return status;
}
int cdflib_tinv(double p, double v, int lowertail, double *x)
{
	int status;
	double q;
	/* Local variables */
	static double fx;
	static double cum, ccum;

	double rtol = cdflib_doubleEps();
	double infinite = cdflib_infinite();
	double mu=0.;
	double sigma=1.;
	double huge=cdflib_doubleHuge();
	int iteration;
	double atol= cdflib_doubleTiny();
	double a, b;
	int changesign=1;

	// P in [0,1]
	status=cdflib_checkp("cdflib_tinv", p, "p");
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Check V
	status=cdflib_tCheckParams("cdflib_tinv", v);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	// Compute the couple (p,q) from (p,lowertail)
	if (lowertail==CDFLIB_LOWERTAIL)
	{
		q=1-p;
	}
	else
	{
		q=p;
		p=1-q;
	}
	if ( p == 0. ) 
	{
		*x = -infinite;
		status = CDFLIB_OK;
		return status;
	}
	if ( q == 0. ) 
	{
		*x = infinite;
		status = CDFLIB_OK;
		return status;
	}
	if (cdflib_isnan(p) || cdflib_isnan(v))
	{
		*x = p+v;
		status = CDFLIB_OK;
		return status;
	}
	if (v==infinite)
	{
		if (p<q)
		{
			status = cdflib_norminv(p, mu, sigma, CDFLIB_LOWERTAIL, x);
		}
		else
		{
			status = cdflib_norminv(q, mu, sigma, CDFLIB_UPPERTAIL, x);
		}
		return status;
	}
	//
	// Step A: Find a rough estimate of an interval.
	// If not, this would make a terrible a=-huge, b=huge.
	// This requires too many iterations (more than 500, for typical cases).
	// Compute a "small" b, so that f(b) changes sign.
	// b has the sequence -1.e308, -1.e298, ...,-0.0179,0.0179,1.79e8,1.79e18,etc...
	// We divide by 10 until we reach zero, then we multiply by 10.
	a=-cdflib_doubleHuge();
	b=a/1.e10;
	iteration=0;
	while (1)
	{
		cdflib_cumt(b, v, &cum, &ccum);
		fx = cdflib_computefx(p, q, cum, ccum);
		if (p <= q && fx>0)
		{
			break;
		}
		if (p > q && fx<0)
		{
			break;
		}
		a=b;
		if (changesign==1 && fabs(b)<0.1)
		{
			b=-b;
			changesign=0;
		}
		else
		{
			if (b<0)
			{
				b=b/1.e10;
			}
			else
			{
				b=b*1.e10;
			}
		}
		iteration=iteration+1;
	}
	// Step B: Refine this estimate.
	/*     .. Get initial approximation for X */
	*x = cdflib_dt1(p, q, v);
	status = 0;
	while (1)
	{
	    zero_rc ( a, b, atol, x, &status, fx );
		if (status<0) 
		{
			break;
		}
		cdflib_cumt(*x, v, &cum, &ccum);
		fx = cdflib_computefx(p, q, cum, ccum);
		if (status==0) 
		{
			break;
		}
		iteration = iteration+1;
	}
	if (status == 0) 
	{
		status = CDFLIB_OK;
	}
	else
	{
		cdflib_unabletoinvert("cdflib_tinv", *x, "x");
		status = CDFLIB_ERROR;
	} 
	cdflib_printiter("cdflib_tinv", iteration);
	return status;
}
