/*
 * Copyright (C) 2012 - Michael Baudin
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h"

int cdflib_gennf(double dfn, double dfd, double xnonc, double *x)
{
	double d__1, d__2;

	static double xden, xnum;
	double tiny=cdflib_doubleTiny();

	/*     JJV changed the code to call cdflib_sgamma and cdflib_snorm directly */
	/*     JJV changed the argument checker to allow DFN = 1.0 */
	/*     JJV in the same way as cdflib_gennch was changed. */
	/*     JJV changed this to call cdflib_sgamma and cdflib_snorm directly */
	if (dfn >= 1.000001f) {
		goto L20;
	}
	/*     JJV case dfn = 1.0 - here I am treating dfn as exactly 1.0 */
	/* Computing 2nd power */
	d__1 = cdflib_snorm() + sqrt(xnonc);
	xnum = d__1 * d__1;
	goto L30;
	/*     JJV case dfn > 1.0 */
L20:
	d__1 = (dfn - 1.) / 2.;
	/* Computing 2nd power */
	d__2 = cdflib_snorm() + sqrt(xnonc);
	xnum = (cdflib_sgamma(d__1) * 2. + d__2 * d__2) / dfn;
L30:
	d__1 = dfd / 2.;
	xden = cdflib_sgamma(d__1) * 2. / dfd;
	if (xden > xnum * tiny) {
		*x = xnum / xden;
		return CDFLIB_OK;
	}
	cdflib_messageprint("Noncentral F Random Number: Generated numbers would cause overflow");
	cdflib_messageprint("Noncentral F Random Number: Returning Nan");
	*x = cdflib_nan();
	return CDFLIB_OK;
}

